@@ -11,6 +11,7 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import ddt
 import mock
 import os
 import six
@@ -23,6 +24,7 @@ from os_brick.initiator import linuxscsi
 from os_brick.tests.initiator import test_connector
 
 
+@ddt.ddt
 class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
 
     def setUp(self):
@@ -138,7 +140,9 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
-    def test_connect_volume(self, get_device_info_mock,
+    @mock.patch.object(base.BaseLinuxConnector, 'check_valid_device')
+    def test_connect_volume(self, check_valid_device_mock,
+                            get_device_info_mock,
                             get_scsi_wwn_mock,
                             remove_device_mock,
                             get_fc_hbas_info_mock,
@@ -146,6 +150,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                             realpath_mock,
                             exists_mock,
                             wait_for_rw_mock):
+        check_valid_device_mock.return_value = True
         get_fc_hbas_mock.side_effect = self.fake_get_fc_hbas
         get_fc_hbas_info_mock.side_effect = self.fake_get_fc_hbas_info
 
@@ -246,9 +251,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                                          devices['devices'][0])
         expected_commands = [
             'multipath -f ' + find_mp_device_path_mock.return_value,
-            'blockdev --flushbufs /dev/sdb',
             'tee -a /sys/block/sdb/device/delete',
-            'blockdev --flushbufs /dev/sdc',
             'tee -a /sys/block/sdc/device/delete',
         ]
         self.assertEqual(expected_commands, self.cmds)
@@ -262,7 +265,9 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
     @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas_info')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
-    def test_connect_volume_multipath_rw(self, get_device_info_mock,
+    @mock.patch.object(base.BaseLinuxConnector, 'check_valid_device')
+    def test_connect_volume_multipath_rw(self, check_valid_device_mock,
+                                         get_device_info_mock,
                                          get_scsi_wwn_mock,
                                          get_fc_hbas_info_mock,
                                          get_fc_hbas_mock,
@@ -271,6 +276,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                                          wait_for_rw_mock,
                                          find_mp_dev_mock):
 
+        check_valid_device_mock.return_value = True
         self._test_connect_volume_multipath(get_device_info_mock,
                                             get_scsi_wwn_mock,
                                             get_fc_hbas_info_mock,
@@ -290,7 +296,9 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
     @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas_info')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
+    @mock.patch.object(base.BaseLinuxConnector, 'check_valid_device')
     def test_connect_volume_multipath_no_access_mode(self,
+                                                     check_valid_device_mock,
                                                      get_device_info_mock,
                                                      get_scsi_wwn_mock,
                                                      get_fc_hbas_info_mock,
@@ -300,6 +308,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                                                      wait_for_rw_mock,
                                                      find_mp_dev_mock):
 
+        check_valid_device_mock.return_value = True
         self._test_connect_volume_multipath(get_device_info_mock,
                                             get_scsi_wwn_mock,
                                             get_fc_hbas_info_mock,
@@ -319,7 +328,9 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
     @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas_info')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
-    def test_connect_volume_multipath_ro(self, get_device_info_mock,
+    @mock.patch.object(base.BaseLinuxConnector, 'check_valid_device')
+    def test_connect_volume_multipath_ro(self, check_valid_device_mock,
+                                         get_device_info_mock,
                                          get_scsi_wwn_mock,
                                          get_fc_hbas_info_mock,
                                          get_fc_hbas_mock,
@@ -328,6 +339,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                                          wait_for_rw_mock,
                                          find_mp_dev_mock):
 
+        check_valid_device_mock.return_value = True
         self._test_connect_volume_multipath(get_device_info_mock,
                                             get_scsi_wwn_mock,
                                             get_fc_hbas_info_mock,
@@ -348,7 +360,9 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
     @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas_info')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
+    @mock.patch.object(base.BaseLinuxConnector, 'check_valid_device')
     def test_connect_volume_multipath_not_found(self,
+                                                check_valid_device_mock,
                                                 get_device_info_mock,
                                                 get_scsi_wwn_mock,
                                                 get_fc_hbas_info_mock,
@@ -358,6 +372,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                                                 wait_for_rw_mock,
                                                 find_mp_dev_mock,
                                                 discover_mp_dev_mock):
+        check_valid_device_mock.return_value = True
         discover_mp_dev_mock.return_value = ("/dev/disk/by-path/something",
                                              None)
 
@@ -401,3 +416,57 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
         expected = []
         actual = self.connector.get_all_available_volumes()
         self.assertItemsEqual(expected, actual)
+
+    @mock.patch('eventlet.greenthread.sleep', mock.Mock())
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_rw')
+    @mock.patch.object(os.path, 'exists', return_value=True)
+    @mock.patch.object(os.path, 'realpath', return_value='/dev/sdb')
+    @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas')
+    @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas_info')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
+    @mock.patch.object(base.BaseLinuxConnector, 'check_valid_device')
+    def test_connect_volume_device_not_valid(self, check_valid_device_mock,
+                                             get_device_info_mock,
+                                             get_scsi_wwn_mock,
+                                             get_fc_hbas_info_mock,
+                                             get_fc_hbas_mock,
+                                             realpath_mock,
+                                             exists_mock,
+                                             wait_for_rw_mock,
+                                             find_mp_dev_mock):
+
+        check_valid_device_mock.return_value = False
+        self.assertRaises(exception.NoFibreChannelVolumeDeviceFound,
+                          self._test_connect_volume_multipath,
+                          get_device_info_mock,
+                          get_scsi_wwn_mock,
+                          get_fc_hbas_info_mock,
+                          get_fc_hbas_mock,
+                          realpath_mock,
+                          exists_mock,
+                          wait_for_rw_mock,
+                          find_mp_dev_mock,
+                          'rw',
+                          True)
+
+    @ddt.data(('/dev/mapper/<WWN>', True),
+              ('/dev/mapper/mpath0', True),
+              ('/dev/disk/by-path/pci-1-fc-1-lun-1', False))
+    @ddt.unpack
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.remove_scsi_device')
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.requires_flush')
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.get_dev_path')
+    def test__remove_devices(self, path_used, was_multipath, get_dev_path_mock,
+                             flush_mock, remove_mock):
+        get_dev_path_mock.return_value = path_used
+        self.connector._remove_devices(mock.sentinel.con_props,
+                                       [{'device': '/dev/sda'}],
+                                       mock.sentinel.device_info)
+        get_dev_path_mock.assert_called_once_with(mock.sentinel.con_props,
+                                                  mock.sentinel.device_info)
+        flush_mock.assert_called_once_with('/dev/sda', path_used,
+                                           was_multipath)
+        remove_mock.assert_called_once_with('/dev/sda',
+                                            flush=flush_mock.return_value)
